# Mustache logic-less templates for Drupal

Integrates Mustache.php plus Mustache.js into Drupal.

Includes a generic framework to build DOM content
which can be automatically synchronized on the client-side.
Any RESTful Json endpoint can be used
for automatic content synchronization.

Using Mustache.php by
 * Copyright (c) 2010-2015 Justin Hileman

Using Mustache.js by
* Copyright (c) 2009 Chris Wanstrath (Ruby)
* Copyright (c) 2010-2014 Jan Lehnardt (JavaScript)
* Copyright (c) 2010-2015 The mustache.js community

More about Mustache: http://mustache.github.io/

## Requirements

The following libraries are required:
- Mustache.php by bobthecow v2.12.0 or newer as vendor library
  https://github.com/bobthecow/mustache.php
- Mustache.js by janl v2.3.0 or newer as Drupal Javascript library
  https://github.com/janl/mustache.js

## Recommendations

The Prepared Data module can help you to build a Json endpoint
for delivering aggregated and expensively generated data.

It's recommended to use a Composer-based workflow to install
any dependency of this module.

## How Mustache templating works in Drupal

The usage of Mustache templates is mainly defined via render arrays.
Therefore, the module provides a special render element called 'mustache'.

Mustache templates can be put besides Twig templates
into the templates directory of your theme. A file which is a Mustache template
must end with .mustache.tpl.

Example: A template for rendering social share buttons could be named
as "social-share-buttons.mustache.tpl" and put into yourtheme/templates/.
To render the Mustache template, a render array can be created as follows:

In case a module wants to define a Mustache template, it can do so by implementing
`hook_mustache_template()`. The Mustache template hooks are documented in the
mustache.api.php file of this module.

```php
$render = [
  '#type' => 'mustache',
  '#template' => 'social_share_buttons',
  '#data' => $data,
];
```

Note that `#template` does not contain the .mustache.tpl file ending.
See also section "Render array examples" below to find out
how to create render arrays for Mustache templates.

## Render array examples

```php
// Renders the template (only) via Mustache.php.
// #data can either be an array or a url pointing to the Json data source.
$render = [
  '#type' => 'mustache',
  '#template' => 'social_share_buttons',
  '#data' => $data,
];

// With #select you can pass a certain subset of nested data to the template.
$nested = ['foo' => ['bar' => ['baz' => 'qux']]];
$render = [
  '#type' => 'mustache',
  '#template' => 'social_share_buttons',
  '#data' => $nested,
  '#select' => ['foo', 'bar'], // Would pass ['baz' => 'qux']
];

// Renders the template via Mustache.php,
// and synchronizes the content via Mustache.js one time per second.
$render = [
  '#type' => 'mustache',
  '#template' => 'social_share_buttons',
  '#data' => $url,
  '#sync' => [
    'period' => 1000,
  ],
];

// Renders a placeholder, which will be replaced by the given
// template via Mustache.js using Json data supplied by $url.
$render = [
  '#type' => 'mustache',
  '#template' => 'social_share_buttons',
  '#data' => $url,
  '#placeholder' => ['#markup' => '<span>Loading...</span>'],
  '#sync' => [
    'wrapper_tag' => 'span', // Set wrapper tag to be a span (default is div).
    'period' => 0, // Synchronization will happen only once.
  ],
];

// Renders the template via Mustache.php using $data, and synchronizes
// the content one time via Mustache.js using Json data supplied by $url.
$render = [
  '#type' => 'mustache',
  '#template' => 'social_share_buttons',
  '#data' => $data,
  '#sync' => [
    'data_url' => $url,
    'period' => 0, // Synchronization will happen only once.
  ],
];
```
