/**
 * @file
 * DOM content synchronization with Mustache templates.
 *
 * Using Mustache.js by
 *   Copyright (c) 2009 Chris Wanstrath (Ruby)
 *   Copyright (c) 2010-2014 Jan Lehnardt (JavaScript)
 *   Copyright (c) 2010-2015 The mustache.js community
 */

window.mustacheSync = window.mustacheSync || {};
mustacheSync.items = mustacheSync.items || [];

(function (Drupal, sync, Mustache, window) {
  sync.dataProviders = sync.dataProviders || {};
  sync.registry = sync.registry || [];
  sync.now = sync.now || function () {
    var item;
    var dataProviders = sync.dataProviders;

    sync.___update = sync.___update || function () {
      var provider = this.provider;
      var update = sync.___update;
      var fetch = sync.___fetch;
      var render = sync.___render;
      if (this.ready !== true) {
        return;
      }
      if (provider.faulty === true) {
        return;
      }
      if (provider.fetching === true) {
        window.setTimeout(update.bind(this), 10);
        return;
      }
      if ((provider.fetched === null) || ((this.period > 0) && (this.period - 1 < Date.now() - provider.fetched))) {
        fetch.call(this);
      }
      else {
        render.call(this);
        if (this.period > 0) {
          // Repeat.
          window.setTimeout(update.bind(this), this.period);
        }
      }
    };
    sync.___fetch = sync.___fetch || function () {
      var provider = this.provider;
      var update = sync.___update;
      var render = sync.___render;
      var request = new XMLHttpRequest();
      provider.fetching = true;

      request.open('GET', provider.url, true);
      request.onload = function () {
        var data;
        if (request.status >= 200 && request.status < 400) {
          try {
            data = JSON.parse(request.responseText);
          }
          catch (e) {
            data = false;
          }
          provider.latest = data;
          provider.fetched = Date.now();
          provider.fetching = false;
          provider.faulty = false;

          render.call(this);
          if (this.period > 0) {
            // Repeat.
            window.setTimeout(update.bind(this), this.period);
          }
        }
        else {
          provider.fetching = false;
          provider.faulty = true;
        }
      }.bind(this);
      request.onerror = function () {
        provider.fetching = false;
        provider.faulty = true;
      };

      request.send();
    };
    sync.___render = sync.___render || function () {
      var el = this.element;
      var data = this.provider.latest;
      var select = this.select;
      var subset = sync.___subset;

      if (select !== null && typeof select === 'object' && data !== null && typeof data === 'object') {
        data = subset(data, select);
      }
      if (typeof data === 'object') {
        el.innerHTML = Mustache.render(this.template, data);
        if (el.classList) {
          el.classList.add('synced');
          el.classList.remove('not-synced');
        }
        else {
          el.className += ' synced';
          el.className = el.className.replace(new RegExp('(^|\\b)not-synced(\\b|$)', 'gi'), ' ');
        }
      }
    };

    while (sync.items.length > 0) {
      item = sync.items.shift();
      if (!dataProviders.hasOwnProperty(item.src)) {
        dataProviders[item.src] = {
          url: item.src,
          latest: {},
          fetched: null,
          fetching: false,
          faulty: false
        };
      }
      item.provider = dataProviders[item.src];
      item.delay = item.delay || 0;
      item.period = item.period || 0;
      item.ready = true;
      sync.registry.push(item);

      window.setTimeout(sync.___update.bind(item), item.delay);
    }
  };
  sync.___subset = sync.___subset || function (data, select) {
    var subset = data;
    var slength = select.length;
    var skey;
    var i;
    for (i = 0; i < slength; i++) {
      skey = select[i];
      if (subset.hasOwnProperty(skey)) {
        subset = subset[skey];
      }
      else {
        return false;
      }
    }
    return subset;
  };

  sync.now();

  Drupal.behaviors.mustacheSync = {
    attach: function attach() {
      sync.now();
    }
  };
}(Drupal, window.mustacheSync, window.Mustache, window));
