<?php

/**
 * @file
 * Hooks for mustache module.
 */

/**
 * @addtogroup hooks
 * @{
 */

/**
 * Register a Mustache template by a module.
 *
 * Themes can't register templates. If a theme contains a template
 * which ends with .mustache.tpl, it will be preferred automatically.
 *
 * @return array
 *   Keyed by template name. Each entry is an array,
 *   with of the following keys:
 *   - file: The uri of the template file.
 *     Mustache templates must have the file ending .mustache.tpl.
 */
function hook_mustache_template() {
  $path = drupal_get_path('module', 'mymodule') . '/templates/';
  return [
    'social_share_buttons' => ['file' => $path . 'social-share-buttons.mustache.tpl'],
  ];
}

/**
 * Alter the registration of Mustache templates.
 *
 * @param array &$registered
 *   An array of registered templates via hook_mustache_template().
 */
function hook_mustache_template_alter(&$registered) {
  $path = drupal_get_path('module', 'othermodule') . '/templates/';
  $registered['social_share_buttons']['file'] = $path . 'some_other_buttons.mustache.tpl';
}

/**
 * @} End of "addtogroup hooks".
 */
